@@ -967,6 +967,18 @@ fido_cred_id_len(const fido_cred_t *cred)
 	return (cred->attcred.id.len);
 }
 
+const unsigned char *
+fido_cred_aaguid_ptr(const fido_cred_t *cred)
+{
+	return (cred->attcred.aaguid);
+}
+
+size_t
+fido_cred_aaguid_len(const fido_cred_t *cred)
+{
+	return (sizeof(cred->attcred.aaguid));
+}
+
 int
 fido_cred_prot(const fido_cred_t *cred)
 {
